@@ -69,6 +69,12 @@ enum
 
 static gint 			ke_log_flags;
 
+typedef struct
+{
+    gint    flag;
+    SREvent *evnt;
+} IdleStruct;    
+    
 #define KE_INTERESTED_MODIFIERS_NUMBER 6
 struct
 {
@@ -190,6 +196,14 @@ static GSList 			   *accessible_key_set_list   = NULL;
 static GSList 			   *ke_user_key_list 	 = NULL;
 static GSList 			   *reg_list = NULL;
 
+/**
+ * Workaround for bug #129951:
+ * Some at-spi key press events are received for four times, 
+   with the same timestamp. A single event with a particular 
+   timestamp value will be fired, the other three will be rejected. 
+ */
+static glong timestamp;
+
 /**
  * keyecho
 **/
@@ -234,6 +248,31 @@ typedef enum
 **/
 static KEStateEnum ke_keyboard_status = KE_IDLE;
 
+IdleStruct* ke_idle_struct_new ()
+{
+    IdleStruct* ke_idle;
+    ke_idle = g_new0 (IdleStruct, 1);
+    return ke_idle;
+}
+
+void ke_idle_struct_free (IdleStruct *ke_idle)
+{
+    sre_release_reference (ke_idle->evnt);
+    g_free (ke_idle);
+}
+
+static gboolean
+ke_repot_layer_cmd (gpointer data)
+{
+    IdleStruct *ke_idle = (IdleStruct*)data;
+
+    if (ke_keyboard_event_sink_cb != NULL) 
+	ke_keyboard_event_sink_cb (ke_idle->evnt, ke_idle->flag);
+	
+    ke_idle_struct_free (ke_idle);
+    return FALSE;
+}
+
 /**
  * ke_log_at_spi
  *
@@ -398,6 +437,7 @@ ke_call_srkey_cb (gulong keyID,
 {
     SREvent *evnt = NULL;
     SRHotkeyData *srhotkey_data = NULL;
+    IdleStruct *ke_idle;
     evnt = sre_new ();
     sru_return_if_fail (evnt);
     srhotkey_data = (SRHotkeyData *) g_new0 (SRHotkeyData, 1);
@@ -424,9 +464,14 @@ ke_call_srkey_cb (gulong keyID,
 	ke_log_gnopernicus (evnt);
 	
 	if (ke_keyboard_event_sink_cb != NULL) 
-	    ke_keyboard_event_sink_cb (evnt,0);
-    }
-    
+	{
+	    sre_add_reference (evnt);
+	    ke_idle = ke_idle_struct_new ();
+	    ke_idle->flag = 0;
+	    ke_idle->evnt = evnt;
+	    g_idle_add (ke_repot_layer_cmd, ke_idle);
+	}
+    }	
     sre_release_reference (evnt);
 }
 
@@ -453,7 +498,11 @@ ke_report_keyecho_event (const AccessibleKeystroke *key, gpointer user_data)
 	     (guint) key->modifiers,
 	     (gint) key->keycode,
 	     (long int) key->timestamp,
-	     key->keystring);   
+	     key->keystring);  
+	     
+    if ((glong) key->timestamp == timestamp)
+	return FALSE;
+    timestamp = (glong) key->timestamp;		      
 
     ke_log_at_spi (key);    
     
@@ -566,6 +615,7 @@ ke_return_key (glong keyID)
 }
 
 
+
 /**
  *
  * ke_call_keyboard_layer_cb
@@ -584,6 +634,7 @@ ke_call_keyboard_layer_cb (const gchar *buf,
 			   SREventType  event_type)
 {
     SREvent *evnt = NULL;
+    IdleStruct *ke_idle;
     
     sru_return_if_fail (buf);
     sru_return_if_fail (strlen(buf) != 0);
@@ -606,9 +657,14 @@ ke_call_keyboard_layer_cb (const gchar *buf,
 
 	/* call event sink from srcore */
         if (ke_keyboard_event_sink_cb != NULL) 
-	    ke_keyboard_event_sink_cb (evnt, flag);
-    }
-	
+	{
+	    sre_add_reference (evnt);
+	    ke_idle = ke_idle_struct_new ();
+	    ke_idle->flag = flag;
+	    ke_idle->evnt = evnt;
+	    g_idle_add (ke_repot_layer_cmd, ke_idle);
+	}
+    }	
     sre_release_reference (evnt);
 }
 
@@ -1247,6 +1303,7 @@ ke_user_key_list_register_events (GSList *list)
 	    AccessibleKeySet 	*keyset = NULL;
 	    gchar		*keysym = NULL;
 
+	    modifiers |= SPI_KEYMASK_NUMLOCK;
 	    /* if no key go to next step */
 	    if (!key) 
 	    {
@@ -1307,19 +1364,19 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 #ifndef USE_ALL_WINDOWS
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON1 | SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_BUTTON1 | SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B1|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON2 | SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_BUTTON2 | SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B2|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON3 | SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_BUTTON3 | SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 			    	      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B3|NUMLOCK)");
@@ -1327,7 +1384,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
 				      SPI_KEYMASK_BUTTON1 | 
 				      SPI_KEYMASK_BUTTON2 |
-				      SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B1|B2|NUMLOCK)");
@@ -1335,7 +1392,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
 				      SPI_KEYMASK_BUTTON1 | 
 				      SPI_KEYMASK_BUTTON3 |
-				      SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B1|B3|NUMLOCK)");
@@ -1343,7 +1400,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
 				      SPI_KEYMASK_BUTTON3 |
 				      SPI_KEYMASK_BUTTON2 |
-				      SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B2|B3|NUMLOCK)");
@@ -1352,7 +1409,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 				      SPI_KEYMASK_BUTTON3 |
 				      SPI_KEYMASK_BUTTON2 |
 				      SPI_KEYMASK_BUTTON1 |
-				      SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(B1|B2|B3|NUMLOCK)");
@@ -1360,7 +1417,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_MOD3,
+				      SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1371,7 +1428,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     ke_print_register_return_value (retval, "(NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_SHIFTLOCK|SPI_KEYMASK_MOD3,
+				      SPI_KEYMASK_SHIFTLOCK|SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1381,7 +1438,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     ke_print_register_return_value (retval, "(SL|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD3,
+				      SPI_KEYMASK_SHIFT|SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1391,7 +1448,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     ke_print_register_return_value (retval, "(S|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_ALT|SPI_KEYMASK_MOD3,
+				      SPI_KEYMASK_ALT|SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS				
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1401,7 +1458,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     ke_print_register_return_value (retval, "(S|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_MOD3,
+				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1411,7 +1468,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     ke_print_register_return_value (retval, "(C|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_ALT|SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD3,
+				      SPI_KEYMASK_ALT|SPI_KEYMASK_SHIFT|SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1421,7 +1478,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     ke_print_register_return_value (retval, "(A|S|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD3,
+				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_SHIFT|SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1431,90 +1488,7 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
     ke_print_register_return_value (retval, "(C|S|NUMLOCK)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_ALT|SPI_KEYMASK_MOD3,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif
-    ke_print_register_return_value (retval, "(C|A|NUMLOCK)");
-    
-
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_MOD2,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif
-
-    ke_print_register_return_value (retval, "(NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_SHIFTLOCK|SPI_KEYMASK_MOD2,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif
-    ke_print_register_return_value (retval, "(SL|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD2,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif
-    ke_print_register_return_value (retval, "(S|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_ALT|SPI_KEYMASK_MOD2,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS				
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif
-    ke_print_register_return_value (retval, "(S|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_MOD2,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif				      
-    ke_print_register_return_value (retval, "(C|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_ALT|SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD2,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif				      
-    ke_print_register_return_value (retval, "(A|S|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_SHIFT|SPI_KEYMASK_MOD2,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-#ifdef USE_ALL_WINDOWS
-				      SPI_KEYLISTENER_ALL_WINDOWS);
-#else
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-#endif
-    ke_print_register_return_value (retval, "(C|S|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_ALT|SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_CONTROL|SPI_KEYMASK_ALT|SPI_KEYMASK_NUMLOCK,
 				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 #ifdef USE_ALL_WINDOWS
 				      SPI_KEYLISTENER_ALL_WINDOWS);
@@ -1561,7 +1535,7 @@ ke_keyecho_register_events (AccessibleKeystrokeListener *ke_key_listener,
     ke_print_register_return_value (retval, "(U)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
 				      (AccessibleKeySet *) ke_key_keyset,
-				      SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_NUMLOCK,
 				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(U|NUMLOCK)");
@@ -1573,7 +1547,7 @@ ke_keyecho_register_events (AccessibleKeystrokeListener *ke_key_listener,
     ke_print_register_return_value (retval, "(SL)");
     retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
 				      (AccessibleKeySet *) ke_key_keyset,
-				      SPI_KEYMASK_SHIFTLOCK | SPI_KEYMASK_MOD2,
+				      SPI_KEYMASK_SHIFTLOCK | SPI_KEYMASK_NUMLOCK,
 				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "(SL|NUMLOCK)");
@@ -1720,7 +1694,8 @@ ke_user_key_list_free (GSList *list)
 {
     GSList *elem = NULL;    
 
-    sru_return_if_fail (list);
+    if (!list)
+	return;
 
     for (elem = list ; elem ; elem = elem->next)
 	g_free (elem->data);
@@ -1892,83 +1867,56 @@ ke_layer_unregister_events (void)
 #ifndef USE_ALL_WINDOWS
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_BUTTON1 |
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_BUTTON2 |
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_BUTTON1 |
 					    SPI_KEYMASK_BUTTON2 |
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_BUTTON1 |
 					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_BUTTON2 |
 					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_BUTTON1 |
 					    SPI_KEYMASK_BUTTON2 |
 					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
 #endif
 
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2);
+					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2 | 
+					    SPI_KEYMASK_NUMLOCK | 
 					    SPI_KEYMASK_SHIFTLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2 | 
+					    SPI_KEYMASK_NUMLOCK | 
 					    SPI_KEYMASK_SHIFT);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2 | 
+					    SPI_KEYMASK_NUMLOCK | 
 					    SPI_KEYMASK_ALT);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2 | 
+					    SPI_KEYMASK_NUMLOCK | 
 					    SPI_KEYMASK_CONTROL);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2  | 
+					    SPI_KEYMASK_NUMLOCK  | 
 					    SPI_KEYMASK_SHIFT |
 					    SPI_KEYMASK_CONTROL);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2  | 
+					    SPI_KEYMASK_NUMLOCK  | 
 					    SPI_KEYMASK_ALT   |
 					    SPI_KEYMASK_CONTROL);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD2  | 
-					    SPI_KEYMASK_SHIFT |
-					    SPI_KEYMASK_ALT);
-
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3 | 
-					    SPI_KEYMASK_SHIFTLOCK);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3 | 
-					    SPI_KEYMASK_SHIFT);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3 | 
-					    SPI_KEYMASK_ALT);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3 | 
-					    SPI_KEYMASK_CONTROL);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3  | 
-					    SPI_KEYMASK_SHIFT |
-					    SPI_KEYMASK_CONTROL);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3  | 
-					    SPI_KEYMASK_ALT   |
-					    SPI_KEYMASK_CONTROL);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_MOD3  | 
+					    SPI_KEYMASK_NUMLOCK  | 
 					    SPI_KEYMASK_SHIFT |
 					    SPI_KEYMASK_ALT);
 }
@@ -1986,12 +1934,12 @@ ke_keyecho_unregister_events (void)
     SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
 					    SPI_KEYMASK_UNMODIFIED );
     SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_MOD2 );
+					    SPI_KEYMASK_NUMLOCK );
     SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
 					    SPI_KEYMASK_SHIFTLOCK );
     SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
 					    SPI_KEYMASK_SHIFTLOCK |
-					    SPI_KEYMASK_MOD2 );
+					    SPI_KEYMASK_NUMLOCK );
 }
 
 /**
