@@ -1511,6 +1511,9 @@ src_speech_process_string (const gchar   *text,
 	
     if (text[0] == ' ' && text[1] == '\0' && smode == SRC_SPEECH_SPELL_NORMAL)
 	return g_strdup (_("space"));
+    if (text[0] == '\n' && text[1] == '\0' && smode == SRC_SPEECH_SPELL_NORMAL)
+	return g_strdup (_("enter"));
+
     if (text[0] && !text[1])
     {
         gint index = src_speech_get_index_for_symbol (crt[0]);
